"""Endpoints for the GTI"""

ENDPOINT_INIT = "gti/public/init"
ENDPOINT_CHECK_NAME = "gti/public/checkName"
ENDPOINT_GET_ROUTE = "gti/public/getRoute"
ENDPOINT_DEPARTURE_LIST = "gti/public/departureList"
ENDPOINT_GET_TARIFF = "gti/public/getTariff"
ENDPOINT_DEPARTURE_COURSE = "gti/public/departureCourse"
ENDPOINT_LIST_STATIONS = "gti/public/listStations"
ENDPOINT_LIST_LINES = "gti/public/listLines"
ENDPOINT_GET_ANNOUNCEMENTS = "gti/public/getAnnouncements"
ENDPOINT_GET_INDIVIDUAL_ROUTE = "gti/public/getIndividualRoute"
ENDPOINT_GET_VEHICLE_MAP = "gti/public/getVehicleMap"
ENDPOINT_GET_TRACK_COORDINATES = "gti/public/getTrackCoordinates"
ENDPOINT_CHECK_POSTAL_CODE = "gti/public/checkPostalCode"
ENDPOINT_GET_STATION_INFORMATION = "gti/public/getStationInformation"
ENDPOINT_TARIFF_ZONE_NEIGHBOURS = "gti/public/tariffZoneNeighbours"
ENDPOINT_TICKET_LIST = "gti/public/ticketList"
